#include "torero.h"

static torero_data internal_data;

void internal_data_set_start_code( char code[] )
{
  extern torero_data internal_data;

  strcpy (internal_data.current_code, code);
}

char *internal_data_get_start_code( void )
{
  extern torero_data internal_data;

  return internal_data.current_code;
}

char *internal_data_get_next_code( void )
{
  extern torero_data internal_data;
  static void *initialized = NULL;

  if ( initialized == NULL ) { /* first call of this function */
    initialized = internal_data_get_start_code ();
    return internal_data.current_code ;
  } else {
    gchar str;
    gchar c1, c2, c3;
    
    c1 = internal_data.current_code[2];
    c2 = internal_data.current_code[1];
    c3 = internal_data.current_code[0];

    c1++;
    if ( c1 > 'Z' ) {
      c1 = 'A';
      c2++;
      if ( c2 > 'Z' ) {
	c2 = 'A';
	c3++;
	if ( c3 > 'Z' ) {
	  torero_printf ("ERROR: not enough possible codes", 0);
	  exit (1);
	}
      }
    }

    internal_data.current_code[2] = c1;
    internal_data.current_code[1] = c2;
    internal_data.current_code[0] = c3;
  }

  return internal_data.current_code;
}

void internal_data_set_from_dir( char dir_name[] )
{
  extern torero_data internal_data;
  gint len;

  len = strlen (dir_name);

  if ( dir_name[len-1] != '/' )
    strcat (dir_name, "/\0");

  strcpy (internal_data.from_dir, dir_name);
}

char *internal_data_get_from_dir( void )
{
  extern torero_data internal_data;

  return internal_data.from_dir;
}

void internal_data_set_to_dir( char dir_name[] )
{
  extern torero_data internal_data;
  gint len;

  len = strlen (dir_name);

  if ( dir_name[len-1] != '/' )
    strcat (dir_name, "/\0");
  
  strcpy (internal_data.to_dir, dir_name);
}

char *internal_data_get_to_dir( void )
{
  extern torero_data internal_data;

  return internal_data.to_dir;
}

void internal_data_set_verbose_level( int level)
{
  extern torero_data internal_data;
  
  internal_data.verbose_level= level;
}

int internal_data_get_verbose_level( void )
{
  extern torero_data internal_data;

  return internal_data.verbose_level;
}

void internal_data_set_up_files_tree( void )
{
  extern torero_data internal_data;

  internal_data.files_tree = g_tree_new (compare_files);
}

GTree *internal_data_get_files_tree( void )
{
  extern torero_data internal_data;

  return internal_data.files_tree;
}

FILE *internal_data_set_up_heads_base_file( void )
{
  extern torero_data internal_data;
  gchar *path;
  gchar file_name[MAX_TORERO_STRLINE];
  gchar message[MAX_TORERO_STRLINE];
  
  path = internal_data_get_to_dir ();

  sprintf (file_name, "%sheads_file.txt", path);
  sprintf (message, "creating file: %s", file_name);
  torero_printf (message, 0);

  internal_data.heads_base_fp = fopen (file_name, "w");

  if ( internal_data.heads_base_fp == NULL ) { /* have we created this file? */
    sprintf (message, "ERROR: unable to create file: %s", file_name);
    torero_printf (message, 0);
    exit (1);
  }   

  return internal_data.heads_base_fp;
}

FILE *internal_data_set_up_coords_base_file( void )
{
  extern torero_data internal_data;
  gchar *path;
  gchar file_name[MAX_TORERO_STRLINE];
  gchar message[MAX_TORERO_STRLINE];

  path = internal_data_get_to_dir ();

  sprintf (file_name, "%scoords_file.txt", path);
  sprintf (message, "creating file: %s", file_name);
  torero_printf (message, 0);
  
  internal_data.coords_base_fp = fopen (file_name, "w");

  if ( internal_data.coords_base_fp == NULL ) { /* have we created this file? */
    sprintf (message, "ERROR: unable to create file: %s", file_name);
    torero_printf (message, 0);
    exit (1);
  }

  return internal_data.coords_base_fp;
}

FILE *internal_data_set_up_notpl_base_file( void )
{
  extern torero_data internal_data;
  gchar *path;
  gchar file_name[MAX_TORERO_STRLINE];
  gchar message[MAX_TORERO_STRLINE];

  path = internal_data_get_to_dir ();

  sprintf (file_name, "%snotpl_file.txt", path);
  sprintf (message, "creating file: %s", file_name);
  torero_printf (message, 0);

  internal_data.notpl_base_fp = fopen (file_name, "w");

   if ( internal_data.notpl_base_fp == NULL ) { /* have we created this file? */
    sprintf (message, "ERROR: unable to create file: %s", file_name);
    torero_printf (message, 0);
    exit (1);
  } 

  return internal_data.notpl_base_fp;
}

void internal_data_destroy( void )
{
  extern torero_data internal_data;

  g_tree_destroy (internal_data.files_tree);

  if ( internal_data.heads_base_fp != NULL )
    fclose (internal_data.heads_base_fp);

  if ( internal_data.coords_base_fp != NULL )
    fclose (internal_data.coords_base_fp);

  if ( internal_data.notpl_base_fp != NULL )
    fclose (internal_data.notpl_base_fp);
}
