#include <glib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#define TORERO_VER "1.01"
#define MAX_TORERO_STRLINE 1000
#define MAX_TORERO_FILENAME 100 /* without path */

typedef struct _torero_data {
  char current_code[4];
  char from_dir[MAX_TORERO_STRLINE];
  char to_dir[MAX_TORERO_STRLINE];

  int verbose_level;

  GTree *files_tree;
  FILE *heads_base_fp;
  FILE *coords_base_fp;
  FILE *notpl_base_fp;
} torero_data;

typedef struct _tree_node {
  gchar *heads_filename;
  gchar *coords_filename;
  gchar *notpl_filename;
} tree_node;

int torero_init( int argc,
		 char ***argv );

void torero_printf( char message[], 
		    int message_level );

void torero_print_help( void );

void torero_print_version_num( void );

gint compare_files( gconstpointer node1,
		    gconstpointer node2 );

gboolean concatenate_files( gpointer key,
			    gpointer val, 
			    gpointer data );

tree_node *tree_node_new (gchar heads_filename[],
			  gchar coords_filename[],
			  gchar notpl_filename[]);

void add_file_to_tree( gchar *heads_filename );

int fgetl( FILE *file, gchar *line, int max_strlen );

gchar *get_heads_line( FILE *fp, gchar head_line[] );
gchar *get_coords_line( FILE *fp, gchar coords_line[] );
gchar *get_notpl_line( FILE *fp, gchar notpl_line[] );

gboolean is_current_code_in_coords_line( gchar *coords_line, 
					 gchar *old_code );
gboolean is_current_code_in_notpl_line( gchar *notpl_line,
					gchar *old_code );

void replace_code_in_heads_line( gchar *head_line, 
				 gchar *code,
				 gchar *old_code_bin);
void replace_code_in_coords_line( gchar *coords_line,
				  gchar *code );
void replace_code_in_notpl_line( gchar *notpl_line,
				 gchar *code);

/* declarations of functions defined in internal_data.c */

void internal_data_set_start_code( char code[] );
char *internal_data_get_start_code( void );
char *internal_data_get_next_code( void );

void internal_data_set_from_dir( char dir_name[] );
char *internal_data_get_from_dir( void );

void internal_data_set_to_dir( char dir_name[] ); 
char *internal_data_get_to_dir( void ); 

void internal_data_set_verbose_level( int level );
int internal_data_get_verbose_level( void );

void internal_data_set_up_files_tree( void );
GTree *internal_data_get_files_tree( void );

FILE *internal_data_set_up_heads_base_file( void );

FILE *internal_data_set_up_coords_base_file( void );

FILE *internal_data_set_up_notpl_base_file( void );

void internal_data_destroy( void );
